#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""Output formatters using PyYAML.
"""

from . import base
from cliff import columns


class YAMLFormatter(base.ListFormatter, base.SingleFormatter):

    def add_argument_group(self, parser):
        pass

    def emit_list(self, column_names, data, stdout, parsed_args):
        # the yaml import is slow, so defer loading until we know we want it
        import yaml

        items = []
        for item in data:
            items.append(
                {n: (i.machine_readable()
                     if isinstance(i, columns.FormattableColumn)
                     else i)
                 for n, i in zip(column_names, item)}
            )
        yaml.safe_dump(items, stream=stdout, default_flow_style=False)

    def emit_one(self, column_names, data, stdout, parsed_args):
        # the yaml import is slow, so defer loading until we know we want it
        import yaml

        for key, value in zip(column_names, data):
            dict_data = {
                key: (value.machine_readable()
                      if isinstance(value, columns.FormattableColumn)
                      else value)
            }
            yaml.safe_dump(dict_data, stream=stdout, default_flow_style=False)



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import argparse
import inspect
import traceback

import autopage.argparse

from . import command


class HelpExit(SystemExit):
    """Special exception type to trigger quick exit from the application

    We subclass from SystemExit to preserve API compatibility for
    anything that used to catch SystemExit, but use a different class
    so that cliff's Application can tell the difference between
    something trying to hard-exit and help saying it's done.
    """


class HelpAction(argparse.Action):

    """Provide a custom action so the -h and --help options
    to the main app will print a list of the commands.

    The commands are determined by checking the CommandManager
    instance, passed in as the "default" value for the action.
    """
    def __call__(self, parser, namespace, values, option_string=None):
        app = self.default
        pager = autopage.argparse.help_pager(app.stdout)
        color = pager.to_terminal()
        autopage.argparse.use_color_for_parser(parser, color)
        with pager as out:
            parser.print_help(out)
            title_hl = ('\033[4m', '\033[0m') if color else ('', '')
            out.write('\n%sCommands%s:\n' % title_hl)
            dists_by_module = command._get_distributions_by_modules()

            def dist_for_obj(obj):
                name = inspect.getmodule(obj).__name__.partition('.')[0]
                return dists_by_module.get(name)

            app_dist = dist_for_obj(app)
            command_manager = app.command_manager
            for name, ep in sorted(command_manager):
                try:
                    factory = ep.load()
                except Exception:
                    out.write('Could not load %r\n' % ep)
                    if namespace.debug:
                        traceback.print_exc(file=out)
                    continue
                try:
                    kwargs = {}
                    fact_args = inspect.getfullargspec(factory.__init__).args
                    if 'cmd_name' in fact_args:
                        kwargs['cmd_name'] = name
                    cmd = factory(app, None, **kwargs)
                    if cmd.deprecated:
                        continue
                except Exception as err:
                    out.write('Could not instantiate %r: %s\n' % (ep, err))
                    if namespace.debug:
                        traceback.print_exc(file=out)
                    continue
                one_liner = cmd.get_description().split('\n')[0]
                dist_name = dist_for_obj(factory)
                if dist_name and dist_name != app_dist:
                    dist_info = ' (' + dist_name + ')'
                    if color:
                        dist_info = '\033[90m%s\033[39m' % dist_info
                else:
                    dist_info = ''
                if color:
                    name = '\033[36m%s\033[39m' % name
                out.write('  %-13s  %s%s\n' % (name, one_liner, dist_info))
        raise HelpExit()


class HelpCommand(command.Command):
    """print detailed help for another command
    """

    def get_parser(self, prog_name):
        parser = super(HelpCommand, self).get_parser(prog_name)
        parser.add_argument('cmd',
                            nargs='*',
                            help='name of the command',
                            )
        return parser

    def take_action(self, parsed_args):
        if parsed_args.cmd:
            try:
                the_cmd = self.app.command_manager.find_command(
                    parsed_args.cmd,
                )
                cmd_factory, cmd_name, search_args = the_cmd
            except ValueError:
                # Did not find an exact match
                cmd = parsed_args.cmd[0]
                fuzzy_matches = [k[0] for k in self.app.command_manager
                                 if k[0].startswith(cmd)
                                 ]
                if not fuzzy_matches:
                    raise
                self.app.stdout.write('Command "%s" matches:\n' % cmd)
                for fm in sorted(fuzzy_matches):
                    self.app.stdout.write('  %s\n' % fm)
                return
            self.app_args.cmd = search_args
            kwargs = {}
            if 'cmd_name' in inspect.getfullargspec(cmd_factory.__init__).args:
                kwargs['cmd_name'] = cmd_name
            cmd = cmd_factory(self.app, self.app_args, **kwargs)
            full_name = (cmd_name
                         if self.app.interactive_mode
                         else ' '.join([self.app.NAME, cmd_name])
                         )
            cmd_parser = cmd.get_parser(full_name)
            pager = autopage.argparse.help_pager(self.app.stdout)
            with pager as out:
                autopage.argparse.use_color_for_parser(cmd_parser,
                                                       pager.to_terminal())
                cmd_parser.print_help(out)
        else:
            action = HelpAction(None, None, default=self.app)
            action(self.app.parser, self.app.options, None, None)
        return 0



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import ctypes
import os
import struct
import sys

# Each edit operation is assigned different cost, such as:
#  'w' means swap operation, the cost is 0;
#  's' means substitution operation, the cost is 2;
#  'a' means insertion operation, the cost is 1;
#  'd' means deletion operation, the cost is 3;
# The smaller cost results in the better similarity.
COST = {'w': 0, 's': 2, 'a': 1, 'd': 3}


def damerau_levenshtein(s1, s2, cost):
    """Calculates the Damerau-Levenshtein distance between two strings.

    The Levenshtein distance says the minimum number of single-character edits
    (i.e. insertions, deletions, swap or substitution) required to change one
    string to the other.
    The idea is to reserve a matrix to hold the Levenshtein distances between
    all prefixes of the first string and all prefixes of the second, then we
    can compute the values in the matrix in a dynamic programming fashion. To
    avoid a large space complexity, only the last three rows in the matrix is
    needed.(row2 holds the current row, row1 holds the previous row, and row0
    the row before that.)

    More details:
        https://en.wikipedia.org/wiki/Levenshtein_distance
        https://github.com/git/git/commit/8af84dadb142f7321ff0ce8690385e99da8ede2f
    """

    if s1 == s2:
        return 0

    len1 = len(s1)
    len2 = len(s2)

    if len1 == 0:
        return len2 * cost['a']
    if len2 == 0:
        return len1 * cost['d']

    row1 = [i * cost['a'] for i in range(len2 + 1)]
    row2 = row1[:]
    row0 = row1[:]

    for i in range(len1):
        row2[0] = (i + 1) * cost['d']

        for j in range(len2):

            # substitution
            sub_cost = row1[j] + (s1[i] != s2[j]) * cost['s']

            # insertion
            ins_cost = row2[j] + cost['a']

            # deletion
            del_cost = row1[j + 1] + cost['d']

            # swap
            swp_condition = ((i > 0) and
                             (j > 0) and
                             (s1[i - 1] == s2[j]) and
                             (s1[i] == s2[j - 1])
                             )

            # min cost
            if swp_condition:
                swp_cost = row0[j - 1] + cost['w']
                p_cost = min(sub_cost, ins_cost, del_cost, swp_cost)
            else:
                p_cost = min(sub_cost, ins_cost, del_cost)

            row2[j + 1] = p_cost

        row0, row1, row2 = row1, row2, row0

    return row1[-1]


def terminal_width(stdout):
    if hasattr(os, 'get_terminal_size'):
        # python 3.3 onwards has built-in support for getting terminal size
        try:
            return os.get_terminal_size().columns
        except OSError:
            return None

    if sys.platform == 'win32':
        return _get_terminal_width_windows(stdout)
    else:
        return _get_terminal_width_ioctl(stdout)


def _get_terminal_width_windows(stdout):
    STD_INPUT_HANDLE = -10
    STD_OUTPUT_HANDLE = -11
    STD_ERROR_HANDLE = -12

    std_to_win_handle = {
        sys.stdin: STD_INPUT_HANDLE,
        sys.stdout: STD_OUTPUT_HANDLE,
        sys.stderr: STD_ERROR_HANDLE}

    std_handle = std_to_win_handle.get(stdout)
    if not std_handle:
        return None

    handle = ctypes.windll.kernel32.GetStdHandle(std_handle)
    csbi = ctypes.create_string_buffer(22)

    res = ctypes.windll.kernel32.GetConsoleScreenBufferInfo(handle, csbi)
    if res:
        (size_x, size_y, cur_pos_x, cur_pos_y, attr,
         left, top, right, bottom, max_size_x, max_size_y) = struct.unpack(
            "hhhhHhhhhhh", csbi.raw)
        return size_x


def _get_terminal_width_ioctl(stdout):
    from fcntl import ioctl
    import termios

    try:
        # winsize structure has 4 unsigned short fields
        winsize = b'\0' * struct.calcsize('hhhh')
        try:
            winsize = ioctl(stdout, termios.TIOCGWINSZ, winsize)
        except IOError:
            return None
        except TypeError:
            # this is raised in unit tests as stdout is sometimes a StringIO
            return None
        winsize = struct.unpack('hhhh', winsize)
        columns = winsize[1]
        if not columns:
            return None
        return columns
    except IOError:
        return None



#!/usr/bin/env python
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from io import StringIO
import yaml

from unittest import mock

from cliff.formatters import yaml_format
from cliff.tests import base
from cliff.tests import test_columns


class TestYAMLFormatter(base.TestBase):

    def test_format_one(self):
        sf = yaml_format.YAMLFormatter()
        c = ('a', 'b', 'c', 'd')
        d = ('A', 'B', 'C', '"escape me"')
        expected = {
            'a': 'A',
            'b': 'B',
            'c': 'C',
            'd': '"escape me"'
        }
        output = StringIO()
        args = mock.Mock()
        sf.emit_one(c, d, output, args)
        actual = yaml.safe_load(output.getvalue())
        self.assertEqual(expected, actual)

    def test_formattablecolumn_one(self):
        sf = yaml_format.YAMLFormatter()
        c = ('a', 'b', 'c', 'd')
        d = ('A', 'B', 'C', test_columns.FauxColumn(['the', 'value']))
        expected = {
            'a': 'A',
            'b': 'B',
            'c': 'C',
            'd': ['the', 'value'],
        }
        args = mock.Mock()
        sf.add_argument_group(args)

        args.noindent = True
        output = StringIO()
        sf.emit_one(c, d, output, args)
        value = output.getvalue()
        print(len(value.splitlines()))
        actual = yaml.safe_load(output.getvalue())
        self.assertEqual(expected, actual)

    def test_list(self):
        sf = yaml_format.YAMLFormatter()
        c = ('a', 'b', 'c')
        d = (
            ('A1', 'B1', 'C1'),
            ('A2', 'B2', 'C2'),
            ('A3', 'B3', 'C3')
        )
        expected = [
            {'a': 'A1', 'b': 'B1', 'c': 'C1'},
            {'a': 'A2', 'b': 'B2', 'c': 'C2'},
            {'a': 'A3', 'b': 'B3', 'c': 'C3'}
        ]
        output = StringIO()
        args = mock.Mock()
        sf.add_argument_group(args)
        sf.emit_list(c, d, output, args)
        actual = yaml.safe_load(output.getvalue())
        self.assertEqual(expected, actual)

    def test_formattablecolumn_list(self):
        sf = yaml_format.YAMLFormatter()
        c = ('a', 'b', 'c')
        d = (
            ('A1', 'B1', test_columns.FauxColumn(['the', 'value'])),
        )
        expected = [
            {'a': 'A1', 'b': 'B1', 'c': ['the', 'value']},
        ]
        args = mock.Mock()
        sf.add_argument_group(args)

        args.noindent = True
        output = StringIO()
        sf.emit_list(c, d, output, args)
        actual = yaml.safe_load(output.getvalue())
        self.assertEqual(expected, actual)



#!/usr/bin/env python
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import argparse
import io
from unittest import mock

from cliff.formatters import shell
from cliff.tests import base
from cliff.tests import test_columns


class TestShellFormatter(base.TestBase):

    def test(self):
        sf = shell.ShellFormatter()
        c = ('a', 'b', 'c', 'd')
        d = ('A', 'B', 'C', '"escape me"')
        expected = 'a="A"\nb="B"\nd="\\"escape me\\""\n'
        output = io.StringIO()
        args = mock.Mock()
        args.variables = ['a', 'b', 'd']
        args.prefix = ''
        sf.emit_one(c, d, output, args)
        actual = output.getvalue()
        self.assertEqual(expected, actual)

    def test_args(self):
        sf = shell.ShellFormatter()
        c = ('a', 'b', 'c', 'd')
        d = ('A', 'B', 'C', '"escape me"')
        expected = 'Xd="\\"escape me\\""\n'
        output = io.StringIO()
        # Parse arguments as if passed on the command-line
        parser = argparse.ArgumentParser(description='Testing...')
        sf.add_argument_group(parser)
        parsed_args = parser.parse_args(['--variable', 'd', '--prefix', 'X'])
        sf.emit_one(c, d, output, parsed_args)
        actual = output.getvalue()
        self.assertEqual(expected, actual)

    def test_formattable_column(self):
        sf = shell.ShellFormatter()
        c = ('a', 'b', 'c')
        d = ('A', 'B', test_columns.FauxColumn(['the', 'value']))
        expected = '\n'.join([
            'a="A"',
            'b="B"',
            'c="[\'the\', \'value\']"\n',
        ])
        output = io.StringIO()
        args = mock.Mock()
        args.variables = ['a', 'b', 'c']
        args.prefix = ''
        sf.emit_one(c, d, output, args)
        actual = output.getvalue()
        self.assertEqual(expected, actual)

    def test_non_string_values(self):
        sf = shell.ShellFormatter()
        c = ('a', 'b', 'c', 'd', 'e')
        d = (True, False, 100, '"esc"', str('"esc"'))
        expected = ('a="True"\nb="False"\nc="100"\n'
                    'd="\\"esc\\""\ne="\\"esc\\""\n')
        output = io.StringIO()
        args = mock.Mock()
        args.variables = ['a', 'b', 'c', 'd', 'e']
        args.prefix = ''
        sf.emit_one(c, d, output, args)
        actual = output.getvalue()
        self.assertEqual(expected, actual)

    def test_non_bash_friendly_values(self):
        sf = shell.ShellFormatter()
        c = ('a', 'foo-bar', 'provider:network_type')
        d = (True, 'baz', 'vxlan')
        expected = 'a="True"\nfoo_bar="baz"\nprovider_network_type="vxlan"\n'
        output = io.StringIO()
        args = mock.Mock()
        args.variables = ['a', 'foo-bar', 'provider:network_type']
        args.prefix = ''
        sf.emit_one(c, d, output, args)
        actual = output.getvalue()
        self.assertEqual(expected, actual)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from cliff.command import Command
from cliff.hooks import CommandHook


class Hooked(Command):
    "A command to demonstrate how the hooks work"

    log = logging.getLogger(__name__)

    def take_action(self, parsed_args):
        self.app.stdout.write('this command has an extension\n')


class Hook(CommandHook):
    """Hook sample for the 'hooked' command.

    This would normally be provided by a separate package from the
    main application, but is included in the demo app for simplicity.

    """

    def get_parser(self, parser):
        print('sample hook get_parser()')
        parser.add_argument('--added-by-hook')
        return parser

    def get_epilog(self):
        return 'extension epilog text'

    def before(self, parsed_args):
        self.cmd.app.stdout.write('before\n')

    def after(self, parsed_args, return_code):
        self.cmd.app.stdout.write('after\n')



#!/usr/bin/env python
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from io import StringIO

from cliff.formatters import value
from cliff.tests import base
from cliff.tests import test_columns


class TestValueFormatter(base.TestBase):

    def test(self):
        sf = value.ValueFormatter()
        c = ('a', 'b', 'c', 'd')
        d = ('A', 'B', 'C', '"no escape me"')
        expected = 'A\nB\nC\n"no escape me"\n'
        output = StringIO()
        sf.emit_one(c, d, output, None)
        actual = output.getvalue()
        self.assertEqual(expected, actual)

    def test_formattable_column(self):
        sf = value.ValueFormatter()
        c = ('a', 'b', 'c', 'd')
        d = ('A', 'B', 'C', test_columns.FauxColumn(['the', 'value']))
        expected = "A\nB\nC\n['the', 'value']\n"
        output = StringIO()
        sf.emit_one(c, d, output, None)
        actual = output.getvalue()
        self.assertEqual(expected, actual)

    def test_list_formatter(self):
        sf = value.ValueFormatter()
        c = ('a', 'b', 'c')
        d1 = ('A', 'B', 'C')
        d2 = ('D', 'E', 'F')
        data = [d1, d2]
        expected = 'A B C\nD E F\n'
        output = StringIO()
        sf.emit_list(c, data, output, None)
        actual = output.getvalue()
        self.assertEqual(expected, actual)

    def test_list_formatter_formattable_column(self):
        sf = value.ValueFormatter()
        c = ('a', 'b', 'c')
        d1 = ('A', 'B', test_columns.FauxColumn(['the', 'value']))
        data = [d1]
        expected = "A B ['the', 'value']\n"
        output = StringIO()
        sf.emit_list(c, data, output, None)
        actual = output.getvalue()
        self.assertEqual(expected, actual)



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""Application base class for displaying data about a single object.
"""
import abc

from . import display


class ShowOne(display.DisplayCommandBase, metaclass=abc.ABCMeta):
    """Command base class for displaying data about a single object.
    """

    @property
    def formatter_namespace(self):
        return 'cliff.formatter.show'

    @property
    def formatter_default(self):
        return 'table'

    @abc.abstractmethod
    def take_action(self, parsed_args):
        """Return a two-part tuple with a tuple of column names
        and a tuple of values.
        """

    def produce_output(self, parsed_args, column_names, data):
        (columns_to_include, selector) = self._generate_columns_and_selector(
            parsed_args, column_names)
        if selector:
            data = list(self._compress_iterable(data, selector))
        self.formatter.emit_one(columns_to_include,
                                data,
                                self.app.stdout,
                                parsed_args)
        return 0

    def dict2columns(self, data):
        """Implement the common task of converting a dict-based object
        to the two-column output that ShowOne expects.
        """
        if not data:
            return ({}, {})
        else:
            return zip(*sorted(data.items()))



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import argparse
import functools

from cliff import command
from cliff.tests import base


class TestCommand(command.Command):
    """Description of command.
    """

    def get_parser(self, prog_name):
        parser = super(TestCommand, self).get_parser(prog_name)
        parser.add_argument(
            'long_help_argument',
            help="Create a NIC on the server.\n"
                 "Specify option multiple times to create multiple NICs. "
                 "Either net-id or port-id must be provided, but not both.\n"
                 "net-id: attach NIC to network with this UUID\n"
                 "port-id: attach NIC to port with this UUID\n"
                 "v4-fixed-ip: IPv4 fixed address for NIC (optional)\n"
                 "v6-fixed-ip: IPv6 fixed address for NIC (optional)\n"
                 "none: (v2.37+) no network is attached\n"
                 "auto: (v2.37+) the compute service will automatically "
                 "allocate a network.\n"
                 "Specifying a --nic of auto or none "
                 "cannot be used with any other --nic value.",
        )
        parser.add_argument(
            'regular_help_argument',
            help="The quick brown fox jumps "
                 "over the lazy dog.",
        )
        parser.add_argument(
            '-z',
            dest='zippy',
            default='zippy-default',
            help='defined in TestCommand and used in TestArgumentParser',
        )
        return parser

    def take_action(self, parsed_args):
        return 42


class TestCommandNoDocstring(command.Command):

    def take_action(self, parsed_args):
        return 42


class TestDescription(base.TestBase):

    def test_get_description_docstring(self):
        cmd = TestCommand(None, None)
        desc = cmd.get_description()
        assert desc == "Description of command.\n    "

    def test_get_description_attribute(self):
        cmd = TestCommand(None, None)
        # Artificially inject a value for _description to verify that it
        # overrides the docstring.
        cmd._description = 'this is not the default'
        desc = cmd.get_description()
        assert desc == 'this is not the default'

    def test_get_description_default(self):
        cmd = TestCommandNoDocstring(None, None)
        desc = cmd.get_description()
        assert desc == ''


class TestBasicValues(base.TestBase):

    def test_get_parser(self):
        cmd = TestCommand(None, None)
        parser = cmd.get_parser('NAME')
        assert parser.prog == 'NAME'

    def test_get_name(self):
        cmd = TestCommand(None, None, cmd_name='object action')
        assert cmd.cmd_name == 'object action'

    def test_run_return(self):
        cmd = TestCommand(None, None, cmd_name='object action')
        assert cmd.run(None) == 42


expected_help_message = """
  long_help_argument    Create a NIC on the server.
                        Specify option multiple times to create multiple NICs.
                        Either net-id or port-id must be provided, but not
                        both.
                        net-id: attach NIC to network with this UUID
                        port-id: attach NIC to port with this UUID
                        v4-fixed-ip: IPv4 fixed address for NIC (optional)
                        v6-fixed-ip: IPv6 fixed address for NIC (optional)
                        none: (v2.37+) no network is attached
                        auto: (v2.37+) the compute service will automatically
                        allocate a network.
                        Specifying a --nic of auto or none cannot be used with
                        any other --nic value.
  regular_help_argument
                        The quick brown fox jumps over the lazy dog.
"""


class TestHelp(base.TestBase):

    def test_smart_help_formatter(self):
        cmd = TestCommand(None, None)
        parser = cmd.get_parser('NAME')
        # Set up the formatter to always use a width=80 so that the
        # terminal width of the developer's system does not cause the
        # test to fail. Trying to mock os.environ failed, but there is
        # an arg to HelpFormatter to set the width
        # explicitly. Unfortunately, there is no way to do that
        # through the parser, so we have to replace the parser's
        # formatter_class attribute with a partial() that passes width
        # to the original class.
        parser.formatter_class = functools.partial(
            parser.formatter_class,
            width=78,
        )
        self.assertIn(expected_help_message, parser.format_help())


class TestArgumentParser(base.TestBase):

    def test_option_name_collision(self):
        cmd = TestCommand(None, None)
        parser = cmd.get_parser('NAME')
        # We should have an exception registering an option with a
        # name that already exists because we configure the argument
        # parser to ignore conflicts but this option has no other name
        # to be used.
        self.assertRaises(
            argparse.ArgumentError,
            parser.add_argument,
            '-z',
        )

    def test_option_name_collision_with_alias(self):
        cmd = TestCommand(None, None)
        parser = cmd.get_parser('NAME')
        # We not should have an exception registering an option with a
        # name that already exists because we configure the argument
        # parser to ignore conflicts and this option can be added as
        # --zero even if the -z is ignored.
        parser.add_argument('-z', '--zero')

    def test_resolve_option_with_name_collision(self):
        cmd = TestCommand(None, None)
        parser = cmd.get_parser('NAME')
        parser.add_argument(
            '-z', '--zero',
            dest='zero',
            default='zero-default',
        )
        args = parser.parse_args(['-z', 'foo', 'a', 'b'])
        self.assertEqual(args.zippy, 'foo')
        self.assertEqual(args.zero, 'zero-default')

    def test_with_conflict_handler(self):
        cmd = TestCommand(None, None)
        cmd.conflict_handler = 'resolve'
        parser = cmd.get_parser('NAME')
        self.assertEqual(parser.conflict_handler, 'resolve')

    def test_raise_conflict_argument_error(self):
        cmd = TestCommand(None, None)
        parser = cmd.get_parser('NAME')
        parser.add_argument(
            '-f', '--foo',
            dest='foo',
            default='foo',
        )
        self.assertRaises(
            argparse.ArgumentError,
            parser.add_argument,
            '-f',
        )

    def test_resolve_conflict_argument(self):
        cmd = TestCommand(None, None)
        cmd.conflict_handler = 'resolve'
        parser = cmd.get_parser('NAME')
        parser.add_argument(
            '-f', '--foo',
            dest='foo',
            default='foo',
        )
        parser.add_argument(
            '-f', '--foo',
            dest='foo',
            default='bar',
        )
        args = parser.parse_args(['a', 'b'])
        self.assertEqual(args.foo, 'bar')

    def test_wrong_conflict_handler(self):
        cmd = TestCommand(None, None)
        cmd.conflict_handler = 'wrong'
        self.assertRaises(
            ValueError,
            cmd.get_parser,
            'NAME',
        )



#!/usr/bin/env python
#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import os
import sys
from unittest import mock

from cliff.tests import base
from cliff import utils


class TestTerminalWidth(base.TestBase):

    def test(self):
        width = utils.terminal_width(sys.stdout)
        # Results are specific to the execution environment, so only assert
        # that no error is raised.
        if width is not None:
            self.assertIsInstance(width, int)

    @mock.patch('cliff.utils.os')
    def test_get_terminal_size(self, mock_os):
        ts = os.terminal_size((10, 5))
        mock_os.get_terminal_size.return_value = ts
        width = utils.terminal_width(sys.stdout)
        self.assertEqual(10, width)
        mock_os.get_terminal_size.side_effect = OSError()
        width = utils.terminal_width(sys.stdout)
        self.assertIs(None, width)



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import unittest

from cliff import columns


class FauxColumn(columns.FormattableColumn):

    def human_readable(self):
        return 'I made this string myself: {}'.format(self._value)


class TestColumns(unittest.TestCase):

    def test_faux_column_machine(self):
        c = FauxColumn(['list', 'of', 'values'])
        self.assertEqual(['list', 'of', 'values'], c.machine_readable())

    def test_faux_column_human(self):
        c = FauxColumn(['list', 'of', 'values'])
        self.assertEqual(
            u"I made this string myself: ['list', 'of', 'values']",
            c.human_readable(),
        )

    def test_sorting(self):
        cols = [
            FauxColumn('foo'),
            FauxColumn('bar'),
            FauxColumn('baz'),
            FauxColumn('foo'),
        ]
        cols.sort()
        self.assertEqual(
            ['bar', 'baz', 'foo', 'foo'],
            [c.machine_readable() for c in cols],
        )



import logging
import os

from cliff.show import ShowOne


class File(ShowOne):
    "Show details about a file"

    log = logging.getLogger(__name__)

    def get_parser(self, prog_name):
        parser = super(File, self).get_parser(prog_name)
        parser.add_argument('filename', nargs='?', default='.')
        return parser

    def take_action(self, parsed_args):
        stat_data = os.stat(parsed_args.filename)
        columns = ('Name',
                   'Size',
                   'UID',
                   'GID',
                   'Modified Time',
                   )
        data = (parsed_args.filename,
                stat_data.st_size,
                stat_data.st_uid,
                stat_data.st_gid,
                stat_data.st_mtime,
                )
        return (columns, data)
